SUCCESS = 0
PATH_NOT_FOUND_ERROR = 1
PATH_NOT_FOUND_ERROR_MSG = "Path not found: {} - error: {}"
PATH_LOAD_ERROR = 2
PATH_LOAD_ERROR_MSG = "Failed to load path: {}"
PATH_CREATE_ERROR = 3
PATH_CREATE_ERROR_MSG = "Failed to create path: {}"
CONFIGURE_IMAGE_ERROR = 4
CONFIGURE_IMAGE_ERROR_MSG = "Failed to configure image: {} - error: {}"
RESET_IMAGE_ERROR = 5
RESET_IMAGE_ERROR_MSG = "Failed to reset image: {} - error: {}"

MISSING_ATTRIBUTE_ERROR = 6
MISSING_ATTRIBUTE_ERROR_MSG = "Missing attribute: {} in {}"
INVALID_ATTRIBUTE_TYPE_ERROR = 7
INVALID_ATTRIBUTE_TYPE_ERROR_MSG = (
    "Invalid attribute type: {} for value: {} - must be {}"
)
CHECKSUM_ERROR = 8
RESIZE_ERROR = 9
RESIZE_ERROR_MSG = "Failed to resize path: {}"
CHECK_ERROR = 10
CHECK_ERROR_MSG = "Failed to check path: {}"
JSON_DUMP_ERROR = 11
JSON_DUMP_ERROR_MSG = "Failed to dump JSON: {}"
DOWNLOAD_ERROR = 12
FUNCTION_NOT_FOUND_ERROR = 13
